@@ -4,9 +4,9 @@ use std::ops::Deref;
 
 use cosmwasm_std::{
     to_binary, Addr, Api, Attribute, BankMsg, Binary, BlockInfo, Coin, ContractInfo,
-    ContractResult, CustomQuery, Deps, DepsMut, Env, Event, MessageInfo, Order, Querier,
-    QuerierWrapper, Reply, ReplyOn, Response, StdResult, Storage, SubMsg, SubMsgExecutionResponse,
-    TransactionInfo, WasmMsg, WasmQuery,
+    ContractInfoResponse, ContractResult, CustomQuery, Deps, DepsMut, Env, Event, MessageInfo,
+    Order, Querier, QuerierWrapper, Reply, ReplyOn, Response, StdResult, Storage, SubMsg,
+    SubMsgExecutionResponse, TransactionInfo, WasmMsg, WasmQuery,
 };
 use cosmwasm_storage::{prefixed, prefixed_read, PrefixedStorage, ReadonlyPrefixedStorage};
 use prost::Message;
@@ -23,21 +23,7 @@ use crate::executor::AppResponse;
 use crate::transactions::transactional;
 use cosmwasm_std::testing::mock_wasmd_attr;
 
-use anyhow::{bail, Result as AnyResult};
-
-// TODO: we should import this from cosmwasm-std, but cannot due to non_exhaustive so copy here
-#[derive(Serialize, Deserialize, Clone, Debug, PartialEq, JsonSchema)]
-pub struct ContractInfoResponse {
-    pub code_id: u64,
-    /// address that instantiated this contract
-    pub creator: String,
-    /// admin who can run migrations (if any)
-    pub admin: Option<String>,
-    /// if set, the contract is pinned to the cache, and thus uses less gas when called
-    pub pinned: bool,
-    /// set if this contract has bound an IBC port
-    pub ibc_port: Option<String>,
-}
+use anyhow::{bail, Context, Result as AnyResult};
 
 // Contract state is kept in Storage, separate from the contracts themselves
 const CONTRACTS: Map<&Addr, ContractData> = Map::new("contracts");
@@ -113,7 +99,7 @@ pub trait Wasm<ExecC, QueryC> {
 pub struct WasmKeeper<ExecC, QueryC> {
     /// code is in-memory lookup that stands in for wasm code
     /// this can only be edited on the WasmRouter, and just read in caches
-    codes: HashMap<usize, Box<dyn Contract<ExecC>>>,
+    codes: HashMap<usize, Box<dyn Contract<ExecC, QueryC>>>,
     /// Just markers to make type elision fork when using it as `Wasm` trait
     _p: std::marker::PhantomData<QueryC>,
 }
@@ -152,13 +138,8 @@ where
             WasmQuery::ContractInfo { contract_addr } => {
                 let addr = api.addr_validate(&contract_addr)?;
                 let contract = self.load_contract(storage, &addr)?;
-                let res = ContractInfoResponse {
-                    code_id: contract.code_id as u64,
-                    creator: contract.creator.to_string(),
-                    admin: contract.admin.map(|x| x.to_string()),
-                    pinned: false,
-                    ibc_port: None,
-                };
+                let mut res = ContractInfoResponse::new(contract.code_id as u64, contract.creator);
+                res.admin = contract.admin.map(|x| x.into());
                 to_binary(&res).map_err(Into::into)
             }
             query => bail!(Error::UnsupportedWasmQuery(query)),
@@ -174,7 +155,11 @@ where
         sender: Addr,
         msg: WasmMsg,
     ) -> AnyResult<AppResponse> {
-        self.execute_wasm(api, storage, router, block, sender, msg)
+        self.execute_wasm(api, storage, router, block, sender.clone(), msg.clone())
+            .context(format!(
+                "error executing WasmMsg:\nsender: {}\n{:?}",
+                sender, msg
+            ))
     }
 
     fn sudo(
@@ -195,7 +180,7 @@ where
 }
 
 impl<ExecC, QueryC> WasmKeeper<ExecC, QueryC> {
-    pub fn store_code(&mut self, code: Box<dyn Contract<ExecC>>) -> usize {
+    pub fn store_code(&mut self, code: Box<dyn Contract<ExecC, QueryC>>) -> usize {
         let idx = self.codes.len() + 1;
         self.codes.insert(idx, code);
         idx
@@ -721,7 +706,7 @@ where
         action: F,
     ) -> AnyResult<T>
     where
-        F: FnOnce(&Box<dyn Contract<ExecC>>, Deps, Env) -> AnyResult<T>,
+        F: FnOnce(&Box<dyn Contract<ExecC, QueryC>>, Deps<QueryC>, Env) -> AnyResult<T>,
     {
         let contract = self.load_contract(storage, &address)?;
         let handler = self
@@ -749,7 +734,7 @@ where
         action: F,
     ) -> AnyResult<T>
     where
-        F: FnOnce(&Box<dyn Contract<ExecC>>, DepsMut, Env) -> AnyResult<T>,
+        F: FnOnce(&Box<dyn Contract<ExecC, QueryC>>, DepsMut<QueryC>, Env) -> AnyResult<T>,
         ExecC: DeserializeOwned,
     {
         let contract = self.load_contract(storage, &address)?;
@@ -794,7 +779,7 @@ where
             .range_raw(storage, None, None, Order::Ascending)
             .count();
         // we make this longer so it is not rejected by tests
-        Addr::unchecked(format!("Contract #{}", count.to_string()))
+        Addr::unchecked(format!("Contract #{}", count))
     }
 
     fn contract_namespace(&self, contract: &Addr) -> Vec<u8> {
@@ -946,7 +931,7 @@ mod test {
         let mut wasm_storage = MockStorage::new();
         let mut keeper = WasmKeeper::new();
         let block = mock_env().block;
-        let code_id = keeper.store_code(error::contract());
+        let code_id = keeper.store_code(error::contract(false));
 
         transactional(&mut wasm_storage, |cache, _| {
             // cannot register contract with unregistered codeId
@@ -1059,13 +1044,8 @@ mod test {
             .query(&api, &wasm_storage, &querier, &block, query)
             .unwrap();
 
-        let expected = ContractInfoResponse {
-            code_id: code_id as u64,
-            creator: "foobar".to_owned(),
-            admin: Some("admin".to_owned()),
-            pinned: false,
-            ibc_port: None,
-        };
+        let mut expected = ContractInfoResponse::new(code_id as u64, "foobar");
+        expected.admin = Some("admin".to_owned());
         assert_eq!(expected, from_slice(&info).unwrap());
     }
 
